/**
 * Main Program of a RC5, rotational encoder and keyboard interface
 * @file Main.c
 * use a P89LPC935 or 936 with 6Mhz resonator
 * communicates at 115200 bit/s
 * 
 * V3 2013-02-07
 * code clean-up
 * V2 2013-01-15
 * code clean-up, switch off after 20 instead of 15 seconds, added code 1 for power-up via RC4
 * V1 2013-01-05
 * leave monitor on during shuntdown down, switch off after 15 instead of 10 seconds
 * V0 2013-01-03
 */

// This file defines registers available in P89LPC93X
#include <p89lpc935_6.h>
#include <stdio.h>

#define DelayStartup	40
#define DelayShutdown	20

#define _RC5inp		P1_7	//define input pin

#define Pin_PowerSwitch	P2_2
#define Pin_AmpOff	P2_3
#define Pin_PiOn		P2_5

#define RC5address	0
	
// Encoder and Keys should not overlap in their bits
#define KeysPort		P0		//select port with keys
#define SelKeys		0b11001100	//select keys within key port
#define KeyPowerMute	0b00000100
#define KeyA		0b00001000
#define KeyB		0b01000000
#define KeyC		0b10000000

#define KeyPressShort	20

void T0_isr(void) __interrupt (1);		//int from Timer 0 to read RC5 state
void T1_isr(void) __interrupt (3);		//int from Timer 1 to read encoder state
void WDT_RTC_isr(void) __interrupt (10);	//int from internal RTC to update PWM read keys

#define isrregisterbank	2		//for all isr on the SAME priority level 

volatile unsigned char KeyState;		//stores the keys state
volatile unsigned char OldKeyState;
volatile unsigned char KeyPressDuration;

__bit AmpOff; 
__bit PiOn;
__bit PowerSwitch;

#include "InitMCU.c"
#include "DualEncoder.c"
#include "RC5.c"

// link stdio to UART
void  putchar(char c) 
{
	SBUF = c;
	while (0==TI){}
	TI = 0;
}

void WDT_RTC_isr(void) __interrupt (10) __using(isrregisterbank)	//int from internal RTC to read keys
{
	//Get KeysState and invert it
	KeyState = ~KeysPort & SelKeys;
	//decode keystate and measure time, no release debouncing, so do not call to often
	if (KeyState)					//Key pressed?
		{
		if (0 == OldKeyState)			//Is this key new?
			{
			OldKeyState = KeyState;		// then store it
			KeyPressDuration = 0;		//tick zero
			}
		else if (OldKeyState == KeyState) 
			{				//same key is pressed
			if (0xFF != KeyPressDuration)	//count ticks while same key is being pressed
				++KeyPressDuration;
			}
		else
			{				//we have a new key without an release of the old one
			OldKeyState = KeyState;		// store new one
			KeyPressDuration=0;		// reset time
			}
		}
	RTCCON = RTCen;					//Reset int flag manually
}

void WaitSeconds(unsigned char seconds)
{
	// use all interupt sources to create delay
	unsigned long j;
	for(j=(unsigned long)seconds*(unsigned long)18463; j; j--)
		{
		PCON=0b00000001;				//go idel, wake up by any int
		}
}

void ToggelPower()
{
	PiOn=!PiOn;

	AmpOff=!PiOn;

	//switch monitor synchronusly
	PowerSwitch=PiOn;

	if (PiOn)
		{					// Power up
		Pin_PiOn=PiOn;
		P1_0=PiOn;
		P1_1=PiOn;
		Pin_PowerSwitch=PowerSwitch;
		WaitSeconds(DelayStartup);
		Pin_AmpOff=AmpOff;
		}
	else
		{					// Power down
		Pin_AmpOff=AmpOff;
		printf_tiny("shutdown\n");
		WaitSeconds(DelayShutdown);
		Pin_PowerSwitch=PowerSwitch;
		P1_0=PiOn;
		P1_1=PiOn;
		Pin_PiOn=PiOn;
		}
}

void ToggelMute()
{
	if (PiOn)			//do not switch the amp on if the pi is off
		{
		AmpOff=!AmpOff;
		Pin_AmpOff=AmpOff;
		}
}


void DecodeRemote()
{
	__bit static RTbitold;				//Togglebit des letzten Befehls von RC5

	if (RC5address==rAddress)
		{
		if (RTbit ^ RTbitold)	//Neue Taste erkannt
			{
			switch (rCommand)
				{
				case 1:
					if(0==PiOn)
						{
						ToggelPower();
						}
					break;
				case 12:			//Standby
					ToggelPower();
					break;
				case 13:			//Mute
					ToggelMute();
					break;
  				}
			}
		if (1 == PiOn)
			{
			printf_tiny("RC5 %d %d\n", rCommand, RTbit ^ RTbitold);
			}
  		RTbitold=RTbit;		//Togglebit speichern
  		}
	rCounter=0;			//Nach Erkennung zurücksetzen
}

/** Main loop */
void main()
{
	InitMCU();
	AmpOff=1;
	// Infinite loop
	while(1) {

		if (12==rCounter)
			{
			DecodeRemote();
			}

		// A Key was pressed if OldKeyState != 0 and Keystate = 0
		// OldKeyState = 0 must be set by receiving program after decoding as a flag
		if (0 == KeyState)
			{
			switch(OldKeyState)
				{
				case KeyPowerMute:
					if (KeyPressShort > KeyPressDuration)
						{
						if (PiOn) 
							{
							ToggelMute();
							}
						else
							{
							ToggelPower();
							}
						}
					else 
						{
						ToggelPower();
						}
				break;
				case KeyA:
				printf_tiny("KeyA %d\n", KeyPressDuration);
				break;
				case KeyB:
				printf_tiny("KeyB %d\n", KeyPressDuration);
				break;
				case KeyC:
				printf_tiny("KeyC %d\n", KeyPressDuration);
				break;
				}
			OldKeyState=0;
			}

		// A Rotation occured if EncoderSteps!= 0
		// EncoderSteps = 0 must be set by receiving program after decoding
		if (EncoderStepsA)
			{
			printf_tiny("EncA %d\n", EncoderStepsA);
			EncoderStepsA = 0;		//ack received steps
			}
		if (EncoderStepsB)
			{
			printf_tiny("EncB %d\n", EncoderStepsB);
			EncoderStepsB = 0;		//ack received steps
			}
		PCON=0b00000001;				//go idel, wake up by any int
	}
}
